@@ -723,7 +723,7 @@ static NTSTATUS fork_and_exec( UNICODE_STRING *path, int unixdir,
     char *unix_name;
     NTSTATUS status;
 
-    status = nt_to_unix_file_name( path, &unix_name, FILE_OPEN );
+    status = nt_to_unix_file_name( path, &unix_name, NULL, FILE_OPEN );
     if (status) return status;
 
 #ifdef HAVE_PIPE2
@@ -893,7 +893,6 @@ NTSTATUS WINAPI NtCreateUserProcess( HANDLE *process_handle_ptr, HANDLE *thread_
 
     TRACE( "%s image %s cmdline %s parent %p\n", debugstr_us( &path ),
            debugstr_us( &params->ImagePathName ), debugstr_us( &params->CommandLine ), parent );
-    if (debug) FIXME( "debug port %p not supported yet\n", debug );
 
     unixdir = get_unix_curdir( params );
 
@@ -942,11 +941,11 @@ NTSTATUS WINAPI NtCreateUserProcess( HANDLE *process_handle_ptr, HANDLE *thread_
     SERVER_START_REQ( new_process )
     {
         req->token          = wine_server_obj_handle( token );
+        req->debug          = wine_server_obj_handle( debug );
         req->parent_process = wine_server_obj_handle( parent );
         req->inherit_all    = !!(process_flags & PROCESS_CREATE_FLAGS_INHERIT_HANDLES);
         req->create_flags   = params->DebugFlags; /* hack: creation flags stored in DebugFlags for now */
         req->socket_fd      = socketfd[1];
-        req->exe_file       = wine_server_obj_handle( file_handle );
         req->access         = process_access;
         req->cpu            = pe_info.cpu;
         req->info_size      = startup_info_size;
@@ -1367,15 +1366,25 @@ NTSTATUS WINAPI NtQueryInformationProcess( HANDLE handle, PROCESSINFOCLASS class
             if (!info) ret = STATUS_ACCESS_VIOLATION;
             else
             {
-                SERVER_START_REQ(get_process_info)
+                HANDLE debug;
+
+                SERVER_START_REQ(get_process_debug_info)
                 {
                     req->handle = wine_server_obj_handle( handle );
-                    if ((ret = wine_server_call( req )) == STATUS_SUCCESS)
-                    {
-                        *(DWORD_PTR *)info = reply->debugger_present ? ~(DWORD_PTR)0 : 0;
-                    }
+                    ret = wine_server_call( req );
+                    debug = wine_server_ptr_handle( reply->debug );
                 }
                 SERVER_END_REQ;
+                if (ret == STATUS_SUCCESS)
+                {
+                    *(DWORD_PTR *)info = ~0ul;
+                    NtClose( debug );
+                }
+                else if (ret == STATUS_PORT_NOT_SET)
+                {
+                    *(DWORD_PTR *)info = 0;
+                    ret = STATUS_SUCCESS;
+                }
             }
         }
         else ret = STATUS_INFO_LENGTH_MISMATCH;
@@ -1388,15 +1397,18 @@ NTSTATUS WINAPI NtQueryInformationProcess( HANDLE handle, PROCESSINFOCLASS class
             if (!info) ret = STATUS_ACCESS_VIOLATION;
             else
             {
-                SERVER_START_REQ(get_process_info)
+                HANDLE debug;
+
+                SERVER_START_REQ(get_process_debug_info)
                 {
                     req->handle = wine_server_obj_handle( handle );
-                    if ((ret = wine_server_call( req )) == STATUS_SUCCESS)
-                    {
-                        *(DWORD *)info = reply->debug_children;
-                    }
+                    ret = wine_server_call( req );
+                    debug = wine_server_ptr_handle( reply->debug );
+                    *(DWORD *)info = reply->debug_children;
                 }
                 SERVER_END_REQ;
+                if (ret == STATUS_SUCCESS) NtClose( debug );
+                else if (ret == STATUS_PORT_NOT_SET) ret = STATUS_SUCCESS;
             }
         }
         else ret = STATUS_INFO_LENGTH_MISMATCH;
@@ -1409,17 +1421,19 @@ NTSTATUS WINAPI NtQueryInformationProcess( HANDLE handle, PROCESSINFOCLASS class
         break;
 
     case ProcessDebugObjectHandle:
-        /* "These are not the debuggers you are looking for." *
-         * set it to 0 aka "no debugger" to satisfy copy protections */
         len = sizeof(HANDLE);
         if (size == len)
         {
             if (!info) ret = STATUS_ACCESS_VIOLATION;
-            else if (!handle) ret = STATUS_INVALID_HANDLE;
             else
             {
-                memset(info, 0, size);
-                ret = STATUS_PORT_NOT_SET;
+                SERVER_START_REQ(get_process_debug_info)
+                {
+                    req->handle = wine_server_obj_handle( handle );
+                    ret = wine_server_call( req );
+                    *(HANDLE *)info = wine_server_ptr_handle( reply->debug );
+                }
+                SERVER_END_REQ;
             }
         }
         else ret = STATUS_INFO_LENGTH_MISMATCH;
@@ -1488,22 +1502,21 @@ NTSTATUS WINAPI NtQueryInformationProcess( HANDLE handle, PROCESSINFOCLASS class
     case ProcessImageFileName:
         /* FIXME: Should return a device path */
     case ProcessImageFileNameWin32:
-        SERVER_START_REQ(get_dll_info)
+        SERVER_START_REQ( get_process_image_name )
         {
-            UNICODE_STRING *image_file_name_str = info;
+            UNICODE_STRING *str = info;
 
             req->handle = wine_server_obj_handle( handle );
-            req->base_address = 0; /* main module */
-            wine_server_set_reply( req, image_file_name_str ? image_file_name_str + 1 : NULL,
+            req->win32  = (class == ProcessImageFileNameWin32);
+            wine_server_set_reply( req, str ? str + 1 : NULL,
                                    size > sizeof(UNICODE_STRING) ? size - sizeof(UNICODE_STRING) : 0 );
             ret = wine_server_call( req );
             if (ret == STATUS_BUFFER_TOO_SMALL) ret = STATUS_INFO_LENGTH_MISMATCH;
-
-            len = sizeof(UNICODE_STRING) + reply->filename_len;
+            len = sizeof(UNICODE_STRING) + reply->len;
             if (ret == STATUS_SUCCESS)
             {
-                image_file_name_str->MaximumLength = image_file_name_str->Length = reply->filename_len;
-                image_file_name_str->Buffer = (PWSTR)(image_file_name_str + 1);
+                str->MaximumLength = str->Length = reply->len;
+                str->Buffer = (PWSTR)(str + 1);
             }
         }
         SERVER_END_REQ;
@@ -1747,10 +1760,58 @@ NTSTATUS WINAPI NtResumeProcess( HANDLE handle )
 /**********************************************************************
  *           NtDebugActiveProcess  (NTDLL.@)
  */
-NTSTATUS WINAPI NtDebugActiveProcess( HANDLE process, HANDLE debug_object )
+NTSTATUS WINAPI NtDebugActiveProcess( HANDLE process, HANDLE debug )
 {
-    FIXME( "(%p %p), stub!\n", process, debug_object );
-    return STATUS_SUCCESS;
+    NTSTATUS ret;
+
+    SERVER_START_REQ( debug_process )
+    {
+        req->handle = wine_server_obj_handle( process );
+        req->debug  = wine_server_obj_handle( debug );
+        req->attach = 1;
+        ret = wine_server_call( req );
+    }
+    SERVER_END_REQ;
+    return ret;
+}
+
+
+/**********************************************************************
+ *           NtRemoveProcessDebug  (NTDLL.@)
+ */
+NTSTATUS WINAPI NtRemoveProcessDebug( HANDLE process, HANDLE debug )
+{
+    NTSTATUS ret;
+
+    SERVER_START_REQ( debug_process )
+    {
+        req->handle = wine_server_obj_handle( process );
+        req->debug  = wine_server_obj_handle( debug );
+        req->attach = 0;
+        ret = wine_server_call( req );
+    }
+    SERVER_END_REQ;
+    return ret;
+}
+
+
+/**********************************************************************
+ *           NtDebugContinue  (NTDLL.@)
+ */
+NTSTATUS WINAPI NtDebugContinue( HANDLE handle, CLIENT_ID *client, NTSTATUS status )
+{
+    NTSTATUS ret;
+
+    SERVER_START_REQ( continue_debug_event )
+    {
+        req->debug  = wine_server_obj_handle( handle );
+        req->pid    = HandleToULong( client->UniqueProcess );
+        req->tid    = HandleToULong( client->UniqueThread );
+        req->status = status;
+        ret = wine_server_call( req );
+    }
+    SERVER_END_REQ;
+    return ret;
 }
 
 
