@@ -248,6 +248,9 @@ static void _check_property( ULONG line, IWbemClassObject *obj, const WCHAR *pro
     case VT_R4:
         trace( "%s: %f\n", wine_dbgstr_w(prop), V_R4(&val) );
         break;
+    case VT_BOOL:
+        trace( "%s: %d\n", wine_dbgstr_w(prop), V_BOOL(&val) );
+        break;
     default:
         break;
     }
@@ -1144,6 +1147,84 @@ static void test_SystemSecurity( IWbemServices *services )
     SysFreeString( class );
 }
 
+static void test_Win32_NetworkAdapter( IWbemServices *services )
+{
+    BSTR wql = SysAllocString( L"wql" ), query = SysAllocString( L"SELECT * FROM Win32_NetworkAdapter" );
+    IEnumWbemClassObject *result;
+    IWbemClassObject *obj;
+    HRESULT hr;
+    DWORD count;
+
+    hr = IWbemServices_ExecQuery( services, wql, query, 0, NULL, &result );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    for (;;)
+    {
+        VARIANT val;
+        CIMTYPE type;
+
+        hr = IEnumWbemClassObject_Next( result, 10000, 1, &obj, &count );
+        if (hr != S_OK) break;
+
+        check_property( obj, L"Description", VT_BSTR, CIM_STRING );
+        check_property( obj, L"DeviceID", VT_BSTR, CIM_STRING );
+        check_property( obj, L"Index", VT_I4, CIM_UINT32 );
+        check_property( obj, L"Name", VT_BSTR, CIM_STRING );
+
+        type = 0xdeadbeef;
+        VariantInit( &val );
+        hr = IWbemClassObject_Get( obj, L"ServiceName", 0, &val, &type, NULL );
+        ok( hr == S_OK, "failed to get service name %08x\n", hr );
+        ok( V_VT( &val ) == VT_BSTR || broken(V_VT( &val ) == VT_NULL) /* win2k8 */,
+            "unexpected variant type 0x%x\n", V_VT( &val ) );
+        ok( type == CIM_STRING, "unexpected type 0x%x\n", type );
+        VariantClear( &val );
+
+        IWbemClassObject_Release( obj );
+    }
+
+    IEnumWbemClassObject_Release( result );
+    SysFreeString( query );
+
+    query = SysAllocString( L"SELECT * FROM Win32_NetworkAdapter WHERE PNPDeviceID LIKE \"PCI\\\\%\"" );
+    hr = IWbemServices_ExecQuery( services, wql, query, 0, NULL, &result );
+    ok( hr == S_OK, "got %08x\n", hr );
+    SysFreeString( query );
+
+    query = SysAllocString( L"SELECT * FROM Win32_NetworkAdapter WHERE PNPDeviceID LIKE \"PCI\\%\"" );
+    hr = IWbemServices_ExecQuery( services, wql, query, 0, NULL, &result );
+    todo_wine ok( hr == WBEM_E_INVALID_QUERY, "got %08x\n", hr );
+    SysFreeString( query );
+    SysFreeString( wql );
+}
+
+static void test_Win32_NetworkAdapterConfiguration( IWbemServices *services )
+{
+    BSTR wql = SysAllocString( L"wql" ), query = SysAllocString( L"SELECT * FROM Win32_NetworkAdapterConfiguration" );
+    IEnumWbemClassObject *result;
+    IWbemClassObject *obj;
+    HRESULT hr;
+    DWORD count;
+
+    hr = IWbemServices_ExecQuery( services, wql, query, 0, NULL, &result );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    for (;;)
+    {
+        hr = IEnumWbemClassObject_Next( result, 10000, 1, &obj, &count );
+        if (hr != S_OK) break;
+
+        check_property( obj, L"Description", VT_BSTR, CIM_STRING );
+        check_property( obj, L"Index", VT_I4, CIM_UINT32 );
+        check_property( obj, L"IPEnabled", VT_BOOL, CIM_BOOLEAN );
+        IWbemClassObject_Release( obj );
+    }
+
+    IEnumWbemClassObject_Release( result );
+    SysFreeString( query );
+    SysFreeString( wql );
+}
+
 static void test_Win32_OperatingSystem( IWbemServices *services )
 {
     BSTR wql = SysAllocString( L"wql" ), query = SysAllocString( L"SELECT * FROM Win32_OperatingSystem" );
@@ -1772,16 +1853,19 @@ static void test_SystemRestore( IWbemServices *services )
     hr = IWbemServices_GetObject( services, class, 0, NULL, &service, NULL );
     if (hr != S_OK)
     {
-        skip( "SystemRestore not available\n" );
+        win_skip( "SystemRestore not available\n" );
         SysFreeString( class );
         return;
     }
 
     check_property( service, L"CreationTime", VT_NULL, CIM_STRING );
     check_property( service, L"Description", VT_NULL, CIM_STRING );
+    if (0) /* FIXME */
+    {
     check_property( service, L"EventType", VT_NULL, CIM_UINT32 );
     check_property( service, L"RestorePointType", VT_NULL, CIM_UINT32 );
     check_property( service, L"SequenceNumber", VT_NULL, CIM_UINT32 );
+    }
 
     method = SysAllocString( L"Enable" );
     sig_in = NULL;
@@ -1821,6 +1905,33 @@ static void test_SystemRestore( IWbemServices *services )
     SysFreeString( class );
 }
 
+static void test_Win32_LogicalDisk( IWbemServices *services )
+{
+    BSTR wql = SysAllocString( L"wql" );
+    BSTR query = SysAllocString( L"SELECT * FROM Win32_LogicalDisk" );
+    IEnumWbemClassObject *result;
+    IWbemClassObject *obj;
+    HRESULT hr;
+    DWORD count;
+
+    hr = IWbemServices_ExecQuery( services, wql, query, 0, NULL, &result );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    for (;;)
+    {
+        hr = IEnumWbemClassObject_Next( result, 10000, 1, &obj, &count );
+        if (hr != S_OK) break;
+
+        check_property( obj, L"Caption", VT_BSTR, CIM_STRING );
+        check_property( obj, L"Name", VT_BSTR, CIM_STRING );
+        IWbemClassObject_Release( obj );
+    }
+
+    IEnumWbemClassObject_Release( result );
+    SysFreeString( query );
+    SysFreeString( wql );
+}
+
 START_TEST(query)
 {
     BSTR path = SysAllocString( L"ROOT\\CIMV2" );
@@ -1871,6 +1982,9 @@ START_TEST(query)
     test_Win32_DiskDrive( services );
     test_Win32_DisplayControllerConfiguration( services );
     test_Win32_IP4RouteTable( services );
+    test_Win32_LogicalDisk( services );
+    test_Win32_NetworkAdapter( services );
+    test_Win32_NetworkAdapterConfiguration( services );
     test_Win32_OperatingSystem( services );
     test_Win32_PhysicalMemory( services );
     test_Win32_PnPEntity( services );
