@@ -72,15 +72,15 @@ static void clipboard_destroy( struct object *obj );
 static const struct object_ops clipboard_ops =
 {
     sizeof(struct clipboard),     /* size */
+    &no_type,                     /* type */
     clipboard_dump,               /* dump */
-    no_get_type,                  /* get_type */
     no_add_queue,                 /* add_queue */
     NULL,                         /* remove_queue */
     NULL,                         /* signaled */
     NULL,                         /* satisfied */
     no_signal,                    /* signal */
     no_get_fd,                    /* get_fd */
-    no_map_access,                /* map_access */
+    default_map_access,           /* map_access */
     default_get_sd,               /* get_sd */
     default_set_sd,               /* set_sd */
     no_get_full_name,             /* get_full_name */
