@@ -97,6 +97,21 @@ const PSID security_high_label_sid = (PSID)&high_label_sid;
 
 static luid_t prev_luid_value = { 1000, 0 };
 
+static const WCHAR token_name[] = {'T','o','k','e','n'};
+
+struct type_descr token_type =
+{
+    { token_name, sizeof(token_name) },   /* name */
+    TOKEN_ALL_ACCESS | SYNCHRONIZE,       /* valid_access */
+    {                                     /* mapping */
+        STANDARD_RIGHTS_READ | TOKEN_QUERY_SOURCE | TOKEN_QUERY | TOKEN_DUPLICATE,
+        STANDARD_RIGHTS_WRITE | TOKEN_ADJUST_SESSIONID | TOKEN_ADJUST_DEFAULT | TOKEN_ADJUST_GROUPS
+        | TOKEN_ADJUST_PRIVILEGES,
+        STANDARD_RIGHTS_EXECUTE | TOKEN_IMPERSONATE | TOKEN_ASSIGN_PRIMARY,
+        TOKEN_ALL_ACCESS
+    },
+};
+
 struct token
 {
     struct object  obj;             /* object header */
@@ -135,22 +150,20 @@ struct group
 };
 
 static void token_dump( struct object *obj, int verbose );
-static struct object_type *token_get_type( struct object *obj );
-static unsigned int token_map_access( struct object *obj, unsigned int access );
 static void token_destroy( struct object *obj );
 
 static const struct object_ops token_ops =
 {
     sizeof(struct token),      /* size */
+    &token_type,               /* type */
     token_dump,                /* dump */
-    token_get_type,            /* get_type */
     no_add_queue,              /* add_queue */
     NULL,                      /* remove_queue */
     NULL,                      /* signaled */
     NULL,                      /* satisfied */
     no_signal,                 /* signal */
     no_get_fd,                 /* get_fd */
-    token_map_access,          /* map_access */
+    default_map_access,        /* map_access */
     default_get_sd,            /* get_sd */
     default_set_sd,            /* set_sd */
     no_get_full_name,          /* get_full_name */
@@ -171,22 +184,6 @@ static void token_dump( struct object *obj, int verbose )
              token->token_id.low_part, token->primary, token->impersonation_level );
 }
 
-static struct object_type *token_get_type( struct object *obj )
-{
-    static const WCHAR name[] = {'T','o','k','e','n'};
-    static const struct unicode_str str = { name, sizeof(name) };
-    return get_object_type( &str );
-}
-
-static unsigned int token_map_access( struct object *obj, unsigned int access )
-{
-    if (access & GENERIC_READ)    access |= TOKEN_READ;
-    if (access & GENERIC_WRITE)   access |= TOKEN_WRITE;
-    if (access & GENERIC_EXECUTE) access |= STANDARD_RIGHTS_EXECUTE;
-    if (access & GENERIC_ALL)     access |= TOKEN_ALL_ACCESS;
-    return access & ~(GENERIC_READ | GENERIC_WRITE | GENERIC_EXECUTE | GENERIC_ALL);
-}
-
 static SID *security_sid_alloc( const SID_IDENTIFIER_AUTHORITY *idauthority, int subauthcount, const unsigned int subauth[] )
 {
     int i;
@@ -463,16 +460,6 @@ ACL *replace_security_labels( const ACL *old_sacl, const ACL *new_sacl )
     return replaced_acl;
 }
 
-/* maps from generic rights to specific rights as given by a mapping */
-static inline void map_generic_mask(unsigned int *mask, const GENERIC_MAPPING *mapping)
-{
-    if (*mask & GENERIC_READ) *mask |= mapping->GenericRead;
-    if (*mask & GENERIC_WRITE) *mask |= mapping->GenericWrite;
-    if (*mask & GENERIC_EXECUTE) *mask |= mapping->GenericExecute;
-    if (*mask & GENERIC_ALL) *mask |= mapping->GenericAll;
-    *mask &= 0x0FFFFFFF;
-}
-
 static inline int is_equal_luid( const LUID *luid1, const LUID *luid2 )
 {
     return (luid1->LowPart == luid2->LowPart && luid1->HighPart == luid2->HighPart);
@@ -1039,7 +1026,7 @@ static unsigned int token_access_check( struct token *token,
                                  unsigned int desired_access,
                                  LUID_AND_ATTRIBUTES *privs,
                                  unsigned int *priv_count,
-                                 const GENERIC_MAPPING *mapping,
+                                 const generic_map_t *mapping,
                                  unsigned int *granted_access,
                                  unsigned int *status )
 {
@@ -1074,7 +1061,7 @@ static unsigned int token_access_check( struct token *token,
     {
         if (priv_count) *priv_count = 0;
         if (desired_access & MAXIMUM_ALLOWED)
-            *granted_access = mapping->GenericAll;
+            *granted_access = mapping->all;
         else
             *granted_access = desired_access;
         return *status = STATUS_SUCCESS;
@@ -1150,8 +1137,7 @@ static unsigned int token_access_check( struct token *token,
             sid = (const SID *)&ad_ace->SidStart;
             if (token_sid_present( token, sid, TRUE ))
             {
-                unsigned int access = ad_ace->Mask;
-                map_generic_mask(&access, mapping);
+                unsigned int access = map_access( ad_ace->Mask, mapping );
                 if (desired_access & MAXIMUM_ALLOWED)
                     denied_access |= access;
                 else
@@ -1166,8 +1152,7 @@ static unsigned int token_access_check( struct token *token,
             sid = (const SID *)&aa_ace->SidStart;
             if (token_sid_present( token, sid, FALSE ))
             {
-                unsigned int access = aa_ace->Mask;
-                map_generic_mask(&access, mapping);
+                unsigned int access = map_access( aa_ace->Mask, mapping );
                 if (desired_access & MAXIMUM_ALLOWED)
                     current_access |= access;
                 else
@@ -1212,25 +1197,24 @@ const SID *token_get_primary_group( struct token *token )
 
 int check_object_access(struct token *token, struct object *obj, unsigned int *access)
 {
-    GENERIC_MAPPING mapping;
+    generic_map_t mapping;
     unsigned int status;
     int res;
 
     if (!token)
         token = current->token ? current->token : current->process->token;
 
-    mapping.GenericAll = obj->ops->map_access( obj, GENERIC_ALL );
+    mapping.all = obj->ops->map_access( obj, GENERIC_ALL );
 
     if (!obj->sd)
     {
-        if (*access & MAXIMUM_ALLOWED)
-            *access = mapping.GenericAll;
+        if (*access & MAXIMUM_ALLOWED) *access = mapping.all;
         return TRUE;
     }
 
-    mapping.GenericRead  = obj->ops->map_access( obj, GENERIC_READ );
-    mapping.GenericWrite = obj->ops->map_access( obj, GENERIC_WRITE );
-    mapping.GenericExecute = obj->ops->map_access( obj, GENERIC_EXECUTE );
+    mapping.read  = obj->ops->map_access( obj, GENERIC_READ );
+    mapping.write = obj->ops->map_access( obj, GENERIC_WRITE );
+    mapping.exec = obj->ops->map_access( obj, GENERIC_EXECUTE );
 
     res = token_access_check( token, obj->sd, *access, NULL, NULL,
                               &mapping, access, &status ) == STATUS_SUCCESS &&
@@ -1374,7 +1358,8 @@ DECL_HANDLER(duplicate_token)
         struct token *token = token_duplicate( src_token, req->primary, req->impersonation_level, sd, NULL, 0, NULL, 0 );
         if (token)
         {
-            reply->new_handle = alloc_handle_no_access_check( current->process, token, req->access, objattr->attributes );
+            unsigned int access = req->access ? req->access : get_handle_access( current->process, req->handle );
+            reply->new_handle = alloc_handle_no_access_check( current->process, token, access, objattr->attributes );
             release_object( token );
         }
         release_object( src_token );
@@ -1451,7 +1436,6 @@ DECL_HANDLER(access_check)
                                                  TOKEN_QUERY,
                                                  &token_ops )))
     {
-        GENERIC_MAPPING mapping;
         unsigned int status;
         LUID_AND_ATTRIBUTES priv;
         unsigned int priv_count = 1;
@@ -1473,14 +1457,8 @@ DECL_HANDLER(access_check)
             return;
         }
 
-        mapping.GenericRead = req->mapping_read;
-        mapping.GenericWrite = req->mapping_write;
-        mapping.GenericExecute = req->mapping_execute;
-        mapping.GenericAll = req->mapping_all;
-
-        status = token_access_check(
-            token, sd, req->desired_access, &priv, &priv_count, &mapping,
-            &reply->access_granted, &reply->access_status );
+        status = token_access_check( token, sd, req->desired_access, &priv, &priv_count, &req->mapping,
+                                     &reply->access_granted, &reply->access_status );
 
         reply->privileges_len = priv_count*sizeof(LUID_AND_ATTRIBUTES);
 
